@@ -1,13 +1,16 @@
-""" Factory functions for extracting from lxml element trees. """
+"""
+Composable functions for extracting data using 
+`lxml <http://lxml.de/>`_.
+"""
 
 from __future__ import absolute_import, unicode_literals, print_function
 import logging
 import wex.py2compat ; assert wex.py2compat
 from six.moves.urllib_parse import urljoin, quote, unquote
-from functools import partial, wraps
+from functools import partial
 from operator import is_, methodcaller
 
-from six.moves import filter, reduce
+from six.moves import reduce
 
 from lxml.etree import XPath, _ElementTree, Element
 from lxml.cssselect import CSSSelector
@@ -15,7 +18,6 @@ from lxml.html import HTMLParser, XHTML_NAMESPACE
 
 from .composed import composable
 from .cache import cached
-from .iterable import flatten
 
 SKIP = object()
 skip = partial(is_, SKIP)
@@ -47,6 +49,10 @@ class WrapsShim(object):
 @composable
 @cached
 def parse(src):
+    """ Returns an element tree create by `LXML <http://lxml.de/>`_. 
+       :param src: A readable object such as a :class:`wex.response.Response`.
+    """
+
     if not hasattr(src, 'read'):
         return src
     charset = src.headers.get_content_charset()
@@ -78,12 +84,42 @@ def base_url(root):
 
 
 def css(expression):
+    """ Returns a :func:`composable <wex.composed.composable>` callable that
+        will select elements defined by a 
+        `CSS selector <http://en.wikipedia.org/wiki/Cascading_Style_Sheets#Selector>`_ 
+        expression.
+
+        :param expression: The CSS selector expression.
+
+        The callable returned accepts a :class:`wex.response.Response`, a
+        list of elements or an individual element as an argument.
+    """
     return parse | CSSSelector(expression)
 
 
 def xpath(expression, namespaces=default_namespaces):
+    """ Returns :func:`composable <wex.composed.composable>` callable that will
+        select elements defined by an 
+        `XPath <http://en.wikipedia.org/wiki/XPath>`_ expression.
+
+        :param expression: The XPath expression.
+        :param namespaces: The namespaces.
+
+        The callable returned accepts a :class:`wex.response.Response`, a
+        list of elements or an individual element as an argument.
+
+        For example:
+
+        .. code-block:: pycon
+
+            >>> from lxml.html import fromstring
+            >>> tree = fromstring('<h1>Hello</h1>')
+            >>> selector = xpath('//h1')
+
+    """
     return parse | XPath(expression, namespaces=namespaces)
 
+
 def maybe_list(f):
     #@wraps(WrapsShim(f))
     def wrapper(src, *args, **kwargs):
@@ -98,8 +134,6 @@ def attrib(name, default=SKIP):
     return maybe_list(methodcaller('get', name, default))
 
 
-
-
 def base_url_join(f):
     #@wraps(WrapsShim(f))
     def wrapper(src, *args, **kwargs):
@@ -118,36 +152,45 @@ src = maybe_list(base_url_join(methodcaller('get', 'src', SKIP)))
 
 
 @composable
-@maybe_list
-def normalize_space(string, __maybe_list_cache__={}):
-    return space_join(string.split())
+def normalize_space(src):
+    """ Return a whitespace normalized version of its input.
 
+    :param src: text or iterable.
 
-def list_filter_join(src):
-    """ Filters empty strings from a list """
-    if isinstance(src, list):
-        return ' '.join(filter(None, src))
-    return src
+    If ``src`` is iterable then a generator will be returned.
+    """
+    if hasattr(src, '__iter__'):
+        return (normalize_space_and_join(s) for s in src)
+    return space_join(src.split())
 
 
 @composable
-@maybe_list
-def text_content(src, __maybe_list_cache__={}):
-    if hasattr(src, 'text_content'):
-        if hasattr(src.tag, '__call__'):
-            # so it must be an HtmlComment (obviously ;p)
-            return ''
-        return src.text_content()
-    return ''
+def normalize_space_and_join(src):
+    """ Return a string of space-normalized text content. """
+    if hasattr(src, '__iter__'):
+        normalized = [normalize_space_and_join(s) for s in src]
+        return space_join((s for s in normalized if s.strip()))
+    return space_join(src.split())
 
-text = text_content | normalize_space | list_filter_join
-text_list = text_content | normalize_space | list
 
-def itertext(*args, **kwargs):
-    @composable
-    def itertext(src):
-        if not isinstance(src, list):
-            src = [src]
-        return flatten(elem.itertext(*args, **kwargs) for elem in src)
-    return itertext
+@composable
+def itertext(src):
+    """ Iterates text from elements.
 
+        :param src: The element or elements to iterate.
+    """
+    if hasattr(src, 'itertext'):
+        # using '*' to exclude comments
+        return (t for t in src.itertext('*'))
+    elif hasattr(src, '__iter__'):
+        return (itertext(s) for s in src)
+    return src
+
+
+#: A :class:`wex.composed.ComposedFunction` that returns the whitespace 
+#: normalized text from one element.
+text = itertext | normalize_space
+
+#: A :class:`wex.composed.ComposedFunction` that returns the whitespace 
+#: normalized text from zero or more elements joined with a space.
+join_text = itertext | normalize_space_and_join
