@@ -1,6 +1,6 @@
 """ Helper functions for things that are iterable """
 
-from functools import partial, wraps
+from functools import partial
 from itertools import chain
 from six import next
 from six.moves import map as map_
@@ -8,15 +8,24 @@ from .composed import composable
 
 
 class ZeroValuesError(ValueError):
-    """ Zero values were found. """
+    """ Zero values were found when at least one was expected. """
 
 
 class MultipleValuesError(ValueError):
-    """ More than one value was found. """
+    """ More than one value was found when one or none were expected. """
 
 
 @composable
 def first(iterable):
+    """ Returns first item from an iterable.
+
+    :param iterable: The iterable.
+
+    If the iterable is empty then ``None`` is returned.
+    """
+    if not hasattr(iterable, '__iter__'):
+        # turns out it isn't iterable after all
+        return iterable
     i = iter(iterable)
     try:
         v0 = next(i)
@@ -27,6 +36,9 @@ def first(iterable):
 
 @composable
 def one(iterable):
+    if not hasattr(iterable, '__iter__'):
+        # turns out it isn't iterable after all
+        return iterable
     i = iter(iterable)
     v0 = first(i)
     try:
@@ -47,8 +59,9 @@ def one_or_none(iterable):
 
 @composable
 def flatten(iterable):
-    return chain.from_iterable(iterable)
+    subiterables = ((flatten(i) if hasattr(i, '__iter__') else (i,)) for i in iterable)
+    return chain.from_iterable(subiterables)
 
 
-def map(func, *args, **kwargs):
+def partial_map(func, *args, **kwargs):
     return composable(partial(map_, func, *args, **kwargs))
